function [ f, convRad_N ] = calcConvFunction( dX, linkerRad, std, linkerType )
% This function calculates the function with that the dye density should be
% convolved in order to compare it with the measured histogram data. This
% function considers of two terms, the localization precision which follows
% Gaussian distribution and the inaccuracy persisting because of the linker
% length. The later one follows the projection of a sphere surface and
% convolved with a line ('linkerType=sphere') which results in a top-hat
% function or can also be set to be a Gaussian with FWHM equalling to the
% linker length ('linkerType=Gaussian'). The resultant function is the
% convolution of the aforementioned two terms. If the effect of the linker 
% into account as a Gaussian function then it calculates the convolution
% analytically instead of a numerical convolution.
% 
% parents: lineDiameter_fit
% children: -
% 
% INPUT VARIABLES
% dX: distance of the sampling points for the convolution
% linkerRad: length of the linkers
% std: standard deviation of the localization precision, can be a vector containing values belonging to each localization
% linkerType: type of distribution associated with the linker length, can be 'sphere' or 'Gaussian'
% 
% OUTPUT VARIABLES:
% f: the convolution function with that the dye density should be convolved
% convRad_N: half the points of the convolution function minus 0.5

stdMax=max(std);

linker_std=linkerRad/(2*sqrt(2*log(2)));

switch linkerType
    case 'sphere'
        % boundaries for the convolution
        t_max=linkerRad+3*stdMax;
    case 'Gaussian'
        t_max=3*(linker_std+stdMax);
    otherwise
        error('Unknows linker type')
end
t_min=-t_max;

% number of sampling points for the convolved function, from the center
% point, on one side
convRad_N=floor(ceil((t_max-t_min)/dX)/2);
% number of sampling points for the convolved function, an odd number
conv_N=2*convRad_N+1;

% sampling points for the convolved function
t=linspace(-convRad_N*dX, convRad_N*dX, conv_N)';

switch linkerType
    case 'sphere'
        if linkerRad~=0
            % convolution of a Gaussian and a top-hat function
            f_array=(-erf((-linkerRad+t)./sqrt(2*std.^2))+erf((linkerRad+t)./sqrt(2*std.^2)))/(2*(2*linkerRad));
        else
            f_array=1./sqrt(2*pi*std.^2).*exp(-(t).^2./(2*std.^2));
        end
    case 'Gaussian'
        f_array=1./sqrt(2*pi*(std+linker_std).^2).*exp(-(t).^2./(2*(std+linker_std).^2));
    otherwise
        error('Unknows linker type')
end

f=mean(f_array,2);

% for testing:
%figure()
%plot(t, f)
%g=1/sqrt(2*pi*std^2)*exp(-t.^2/(2*std^2));
%plot(t, f, t, g)
%sum(f)*dX  % the area under the concolved function and should be unit
%sum(g)*dX

end

