function [ modelFunc_Y ] = calcModelFunc( convRad_N, modelFuncType, modelFunc_sampling_X, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize, densFuncMax, densFuncBG, pos, varargin )
% This function calculates the model function be fit on the histogram data
% of the measurement. First calculates the function values of the supposed
% dye density, then adds the constant background. In case of 'Gaussian' and
% 'Lorentzian' binding site density functions, the output values are
% calculated by sampling these functions at the given sampling
% coordinates while in case of 'Circle', 'Disk' and 'Rectangular'
% functions, the output values are calculated by integrating functions
% around the the given sampling coordinates. In case of the 'Gaussian',
% 'Lorentzian' and the 'Disk' function, the model funtion width is the full
% width at half maximum, while in case of 'Rectangular' and 'Circle' model
% function, it is the full width.
% 
% parents: calcModelFuncConvolved
% children: -
% 
% INPUT VARIABLES
% convRad_N: half the points of the convolution function minus 0.5
% modelFuncType: model function of the supposed binding site density
% modelFunc_sampling_X: sampling point coordinates of the model function
% modelFunc_sampBoundaries_left: left boundary coordinates of the sampling bins
% modelFunc_sampBoundaries_right: right boundary coordinates of the sampling bins
% densFuncSize: the FWHM (full width at half-maximum) of the supposed binding site density
% densFuncMax: value of the binding site density function at its central position (maximum for every function except the "circle" model function)
% densFuncBG: constant background of the binding site density function
% pos: central position of the binding site density function


    function modelFunc_Y=Rectangular(convRad_N, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize, densFuncMiddle, densFuncSides, pos)
        % Sampling the rectangular function by integrating it over the
        % sampling regions.
        % Good approximation for the convolution if the convolution
        % function does not change much within the sampling regions. Taking
        % its values at the sampling points does not work for this kind os
        % function, the iteration might not be able to step as the output
        % value would not change always in case of small step size.
        
        
        % dye density following top-hat distribution
        R_modelFunc_Y_init=ones(size(modelFunc_sampBoundaries_left))*densFuncSides;
        hatSize=densFuncSize/2;
        
        % left and right bounds of the rectangle
        bound_left=-hatSize+pos;
        bound_right=hatSize+pos;
        
        % sampling values where the sampling region is fully within the
        % rectangle
        R_modelFunc_Y_init(modelFunc_sampBoundaries_left>=bound_left & modelFunc_sampBoundaries_right<=bound_right)=densFuncMiddle;
        
        % left and right sampling region indices where the sampling regions
        % are only partly within the rectangle
        R_boundIdx_left=find(modelFunc_sampBoundaries_left<bound_left & modelFunc_sampBoundaries_right>bound_left);
        R_boundIdx_right=find(modelFunc_sampBoundaries_left<bound_right & modelFunc_sampBoundaries_right>bound_right);
        
        % sampling values where the sampling region is only partly within
        % the rectangle
        if R_boundIdx_left~=R_boundIdx_right
            R_modelFunc_Y_init(R_boundIdx_left)=...
                (bound_left-modelFunc_sampBoundaries_right(R_boundIdx_left))/...
                (modelFunc_sampBoundaries_left(R_boundIdx_left)-modelFunc_sampBoundaries_right(R_boundIdx_left))*(densFuncMiddle-densFuncSides)+densFuncSides;
            R_modelFunc_Y_init(R_boundIdx_right)=...
                (modelFunc_sampBoundaries_left(R_boundIdx_right)-bound_right)/...
                (modelFunc_sampBoundaries_left(R_boundIdx_right)-modelFunc_sampBoundaries_right(R_boundIdx_right))*(densFuncMiddle-densFuncSides)+densFuncSides;
        else
            R_modelFunc_Y_init(R_boundIdx_right)=...
                (bound_left-bound_right)/...
                (modelFunc_sampBoundaries_left(R_boundIdx_right)-modelFunc_sampBoundaries_right(R_boundIdx_right))*(densFuncMiddle-densFuncSides)+densFuncSides;
        end
        
        % the sampling values of the model function
        modelFunc_Y=[ones(convRad_N,1)*densFuncSides;R_modelFunc_Y_init;ones(convRad_N,1)*densFuncSides];
    end


%% Model function prior the convolution

% calculating the values of the selected model function prior to the convolution
switch modelFuncType
    case {'Circle', 'Disk'}
        
        circleRad=densFuncSize/2;
        
        % angle value for the boundary points outside the circle, should be
        % the same for the "left" and "right" boundary points
        thetaOutsideValue=0;       % let it be
        
        % finding which left boundaries are winthin the circle
        sampXWithinBool_left=false(size(modelFunc_sampBoundaries_left));
        sampXWithinBool_left(modelFunc_sampBoundaries_left>-circleRad+pos & modelFunc_sampBoundaries_left<circleRad+pos)=true;
        
        % polar angle for the boundray points
        theta_left(~sampXWithinBool_left,1)=thetaOutsideValue;
        theta_left(sampXWithinBool_left,1)=acos((modelFunc_sampBoundaries_left(sampXWithinBool_left)-pos)/circleRad);
        
        % finding which right boundaries are within the circle
        sampXWithinBool_right=false(size(modelFunc_sampBoundaries_right));
        sampXWithinBool_right(modelFunc_sampBoundaries_right>-circleRad+pos & modelFunc_sampBoundaries_right<circleRad+pos)=true;
        
        % polar angle for the boundray points
        theta_right(~sampXWithinBool_right,1)=thetaOutsideValue;
        theta_right(sampXWithinBool_right,1)=acos((modelFunc_sampBoundaries_right(sampXWithinBool_right)-pos)/circleRad);
        
        % boolean vector for the sampling points within the circle
        sampXWithinBool=sampXWithinBool_left & sampXWithinBool_right;
        
        % indices of the left and right sampling points of the circle where
        % the sampling region is only partly within the circle
        boundIdx_left=find(~sampXWithinBool_left & sampXWithinBool_right);
        boundIdx_right=find(sampXWithinBool_left & ~sampXWithinBool_right);
        
        modelFunc_Y_init=zeros(size(modelFunc_sampBoundaries_left));
        switch modelFuncType
            case 'Circle'
                % sampling values where the sampling region is fully within the
                % rectangle
                modelFunc_Y_init(sampXWithinBool)=circleRad*2*(theta_left(sampXWithinBool)-theta_right(sampXWithinBool));
                % sampling values where the sampling region is only partly within
                % the rectangle
                if boundIdx_left~=boundIdx_right
                    modelFunc_Y_init(boundIdx_left)=circleRad*2*abs(pi-theta_right(boundIdx_left));
                    modelFunc_Y_init(boundIdx_right)=circleRad*2*abs(theta_left(boundIdx_right)-0);
                else
                    modelFunc_Y_init(boundIdx_right)=circleRad*2*pi;
                end
                
                % the sampling values of the model function
                modelFunc_Y=densFuncMax/2/max(modelFunc_Y_init)*[zeros(convRad_N,1);modelFunc_Y_init;zeros(convRad_N,1)];
                
%                 modelFunc_Y_init(modelFunc_X>-circleRad & modelFunc_X<circleRad)=...
%                     densFuncMax*sqrt(1+2*modelFunc_X(modelFunc_X>-circleRad & modelFunc_X<circleRad).^2./...
%                     (circleRad^2-modelFunc_X(modelFunc_X>-circleRad & modelFunc_X<circleRad).^2));
            case 'Disk'
                % sampling values where the sampling region is fully within the
                % rectangle
                modelFunc_Y_init(sampXWithinBool)=circleRad^2*((theta_left(sampXWithinBool)-sin(2*theta_left(sampXWithinBool))/2)-(theta_right(sampXWithinBool)-sin(2*theta_right(sampXWithinBool))/2));
                % sampling values where the sampling region is only partly within
                % the rectangle
                if boundIdx_left~=boundIdx_right
                    modelFunc_Y_init(boundIdx_left)=circleRad^2*(pi-(theta_right(boundIdx_left)-sin(2*theta_right(boundIdx_left))/2));
                    modelFunc_Y_init(boundIdx_right)=circleRad^2*(theta_left(boundIdx_right)-sin(2*theta_left(boundIdx_right))/2);
                else
                    modelFunc_Y_init(boundIdx_right)=circleRad^2*pi;
                end
                
                % the sampling values of the model function
                modelFunc_Y=densFuncMax/max(modelFunc_Y_init)*[zeros(convRad_N,1);modelFunc_Y_init;zeros(convRad_N,1)];
                
%                 modelFunc_Y_init(modelFunc_X>-diskRad & modelFunc_X<diskRad)=...
%                     densFuncMax/diskRad*sqrt(diskRad^2-modelFunc_X(modelFunc_X>-diskRad & modelFunc_X<diskRad).^2);
        end
    case 'Gaussian'
        % Sampling the Gaussian function by taking its values at the
        % sampling points. Good approximation for the Gaussian. Integrating
        % it over the sampling regions would be overcomplication.
        
        % dye density following Gaussian distribution
        GaussSTD=densFuncSize/2.3548;
        modelFunc_Y=densFuncMax*exp(-(modelFunc_sampling_X-pos).^2/(2*GaussSTD^2));
    case 'Lorentzian'
        % Sampling the Lorentzian function by taking its values at the
        % sampling points. No ide how good this approxiamtion is, but
        % should be sufficient if it works well for the Gaussian.
        
        % dye density following Lorentzian distribution
        gamma=densFuncSize/2;
        modelFunc_Y=densFuncMax*1./(1+((modelFunc_sampling_X-pos)/gamma).^2);
    case 'Rectangular'
        modelFunc_Y=Rectangular(convRad_N, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize, densFuncMax, 0, pos);
    case 'Rectangular gap'
        
        densFuncSides=varargin{1};
        rimLeftWidth=varargin{2};
        rimLeftHeight=varargin{3};
        rimLeftPos=varargin{4};
        rimRightWidth=varargin{5};
        rimRightHeight=varargin{6};
        rimRightPos=varargin{7};
        
        modelFunc_Y_temp=Rectangular(convRad_N, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, densFuncSize, densFuncMax, densFuncSides, pos);
        modelFunc_Y_rimLeft=Rectangular(convRad_N, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, rimLeftWidth, rimLeftHeight, 0, rimLeftPos);
        modelFunc_Y_rimRight=Rectangular(convRad_N, modelFunc_sampBoundaries_left, modelFunc_sampBoundaries_right, rimRightWidth, rimRightHeight, 0, rimRightPos);
        
        modelFunc_Y=modelFunc_Y_temp+modelFunc_Y_rimLeft+modelFunc_Y_rimRight;
        
end

% adding the constant background to the model function
modelFunc_Y=modelFunc_Y+densFuncBG;


end

